#include <SGL.H>
#include "neopop.h"
#include "RB_were_here.h"

unsigned char sound_enable, z80_enable;
unsigned int frame;


/*! Get a string that may possibly be translated */
char* system_get_string(STRINGS string_id) {
    return "no strings yet";    
}

/*! Used to generate a critical message for the user. After the message
	has been displayed, the function should return. The message is not
	necessarily a fatal error. */
	
//	void __cdecl system_message(char* vaMessage,...);
void system_message(char* vaMessage,...) {}


void update_inputs()
{
   	//Clear the controller status
	unsigned char up, down, left, right, button_a, button_b, option;
	Uint16 data = Smpc_Peripheral[0].data;
	
   	//Clear the controller status
	up = down = left = right = button_a = button_b = option = 0;

        // buttons
    if((data & PER_DGT_TA) == 0)
        button_a = 1; 
    if((data & PER_DGT_TB) == 0)
        button_b = 1; 
    if((data & PER_DGT_TC) == 0)
        option = 1; 
        // D-pad
    if((data & PER_DGT_KL) == 0)
        left = 1; 
    if((data & PER_DGT_KR) == 0)
        right = 1; 
    if((data & PER_DGT_KU) == 0)
        up = 1; 
    if((data & PER_DGT_KD) == 0)
        down = 1; 
	   //Write the controller status to memory
	ram[0x6F82] = up | (down << 1) | (left << 2) | (right << 3) | 
		(button_a << 4) | (button_b << 5) | (option << 6);

    // framerate estimate and throttle??
    if(((data & PER_DGT_TL) == 0) && (system_frameskip_key > 0))
        system_frameskip_key--;
    if(((data & PER_DGT_TR) == 0) && (system_frameskip_key < 30))
        system_frameskip_key++;
    // sound enable/disable
    if((data & PER_DGT_TY) == 0) {
        sound_enable = 1;
        z80_enable = 1;
    }     
    if((data & PER_DGT_TX) == 0)
        sound_enable = 0; 
    // z80 enable/disable
    if(((data & PER_DGT_TX) == 0) && ((data & PER_DGT_TY) == 0))
        z80_enable = 0;     
}


void update_graphics()
{
    int x, y;
	_u16* row = cfb; //(_u16 *)translate_address_write(0x9000); // <- pattern // map(0x9000);//cfb;
    Uint16 *VRAM = (Uint16 *)VDP2_VRAM_A0;
//    Uint16 data = Smpc_Peripheral[0].data;

#if 1
    //16-bit Frame buffer: Format X4B4G4R4
    for (y = 0; y < SCREEN_HEIGHT; y++) {
        for (x = 0; x < SCREEN_WIDTH; x++) {
            // B G R sample from SL_DEF.H:
            // CD_Blue		(31 << 10) | ( 0 << 5) | (0)  | RGB_Flag
// endianess:    
            // XBGR is swapped GRXB -> BGR
            // !!!! correct !!!
            VRAM[x] = (((row[x] & 0xF00) >> (8 - 1)) | ((row[x] & 0xF000) >> (12-5 - 1)) | ((row[x] & 0x00F) << (10 + 1))) | RGB_Flag;
#if 0
            if((data & PER_DGT_TC) == 0) {
                // XBGR(4444) -> BGR(1555)
                VRAM[x] = (((row[x] & 0xF00) << (10-8 + 1)) | ((row[x] & 0x0F0) << (5 - 4 + 1)) | ((row[x] & 0x00F) << (0 + 1))) | RGB_Flag;
            } else if((data & PER_DGT_ST) == 0) {
                // RGBX(4444) -> BGR(1555)
                // totally wrong colors: no blue included-> green & yellow image
                VRAM[x] = (((row[x] & 0xF000) >> (12 - 1)) | ((row[x] & 0x0F00) >> (8 - 5 - 1)) | ((row[x] & 0x00F0) << (10-4 + 1))) | RGB_Flag;
            } else if((data & PER_DGT_TY) == 0) {
                // XBGR is swapped GRXB -> BGR
                // !!!! correct !!!
                VRAM[x] = (((row[x] & 0xF00) >> (8 - 1)) | ((row[x] & 0xF000) >> (12-5 - 1)) | ((row[x] & 0x00F) << (10 + 1))) | RGB_Flag;
            } else {  
                // not enough green
                // (assume)XRGB -> BGR
                VRAM[x] = (((row[x] & 0xF00) >> (8 - 1)) | ((row[x] & 0x0F0) << (5 - 4 + 1)) | ((row[x] & 0x00F) << (10 + 1))) | RGB_Flag;
            }    
#endif
        }
		VRAM += 512;
		row += SCREEN_WIDTH;
	}
#else	
    for (y = 0; y < 240; y++) {
        for (x = 0; x < 320; x++) {
            // B G R sample from SL_DEF.H:
            // CD_Blue		(31 << 10) | ( 0 << 5) | (0)  | RGB_Flag
//            *(VRAM + y*512 + x) = cfb[y*320 + x] | RGB_Flag;
            VRAM[x] = (((row[x] & 0xF00) >> (8 - 1)) | ((row[x] & 0x0F0) << (5 - 4 + 1)) | ((row[x] & 0x00F) << (10 + 1))) | RGB_Flag;
        }
		VRAM += 512;
		row += 320;
	}
#endif
}

void printMemory16(unsigned int start, unsigned char x, unsigned char y, unsigned char w, unsigned char h)
{
    unsigned char i, j;
    char str[41];
    unsigned short adr = start;
    
    for(i = 0; i < h; i++) {
        for(j = 0; j < w; j++) {
            snprintf(str, 40, "%x", loadW(adr));
            slPrint(str, slLocate(x + j * 6, y + i));
            adr += 2;
        }    
    }    
}


#include "Z80_interface.h"
#include "TLCS900h_registers.h"
#include "TLCS900h_disassemble.h"
#include "TLCS900h_interpret.h"
#include "gfx.h"
#include "interrupt.h"
extern unsigned int cycle;

void printDebug()
{
    char str[41], *str2;
    _u32 pc_temp;
    
    // debug info:
    slPrint("debug info:", slLocate(5, 5));
        // cartridge name
    snprintf(str, 40, "running cart: %s", rom.name);
    slPrint(str, slLocate(5, 6));
    snprintf(str, 40, "of size: %i at %x", rom.length, rom.data);
    slPrint(str, slLocate(5, 7));
        // cycle count
        // register status
//    snprintf(str, 40, "PS %x A %x SP %x", regs.PS, regs.A, regs.SP);
//    slPrint(str, slLocate(5, 8));
// TLCS900h_disassemble():
//Disassembles a single instruction from 'pc', 
//pc is incremented to the start of the next instruction.
#if 0
    pc_temp = pc;
    str2 = TLCS900h_disassemble();
    slPrint(str2, slLocate(1, 10));
    slPrint(str2 + 25, slLocate(20, 10));
    pc = pc_temp;
#endif    
//    snprintf(str, 40, "opcode %x", first);
//    slPrint(str, slLocate(5, 13));
    
    snprintf(str, 40, "pc data: %02X %02X | %02X %02X %02X %02X | %02X %02X", loadB(pc-2), loadB(pc-1), loadB(pc), loadB(pc+1), loadB(pc+2), loadB(pc+3), loadB(pc+4), loadB(pc+5));
    slPrint(str, slLocate(1, 11));

//    snprintf(str, 40, "at 0x6f91 is: %x", loadB(0x6f91));
//    slPrint(str, slLocate(5, 15));

    snprintf(str, 40, "start   PC %08X cycle: %i", rom_header->startPC, cycle);
    slPrint(str, slLocate(1, 18));

    snprintf(str, 40, "TLCS900 PC %08X", pc);
    slPrint(str, slLocate(1, 20));
    snprintf(str, 40, "Z80 PC %08X", Z80_getReg(Z80_REG_PC));
    slPrint(str, slLocate(20, 20));
    
    snprintf(str, 40, "XWA %08X", gprBank[0][0]);
    slPrint(str, slLocate(1, 22));
    snprintf(str, 40, "XBC %08X", gprBank[0][1]);
    slPrint(str, slLocate(1, 23));
    snprintf(str, 40, "XDE %08X", gprBank[0][2]);
    slPrint(str, slLocate(1, 24));
    snprintf(str, 40, "XHL %08X", gprBank[0][3]);
    slPrint(str, slLocate(1, 25));

    snprintf(str, 40, "regL(0) %08X", regL(0));
    slPrint(str, slLocate(18, 22));
    snprintf(str, 40, "regL(1) %08X", regL(1));
    slPrint(str, slLocate(18, 23));
    snprintf(str, 40, "regL(2) %08X", regL(2));
    slPrint(str, slLocate(18, 24));
    snprintf(str, 40, "regL(3) %08X", regL(3));
    slPrint(str, slLocate(18, 25));

    snprintf(str, 40, "XIX %08X", gpr[0]);
    slPrint(str, slLocate(1, 26));
    snprintf(str, 40, "XIY %08X", gpr[1]);
    slPrint(str, slLocate(1, 27));
    snprintf(str, 40, "XIZ %08X", gpr[2]);
    slPrint(str, slLocate(1, 28));
    snprintf(str, 40, "XSP %08X", gpr[3]);
    slPrint(str, slLocate(1, 29));

    snprintf(str, 40, "%c %c %c %c %c %c", (FLAG_S ? 'S' : 's'), (FLAG_Z ? 'Z' : 'z'), (FLAG_H ? 'H' : 'h'), (FLAG_V ? 'V' : 'v'), (FLAG_N ? 'N' : 'n'), (FLAG_C ? 'C' : 'c') );
    slPrint(str, slLocate(18, 26));



//    snprintf(str, 40, "XWA %x XBC %x", regL(0), regL(1));
//    slPrint(str, slLocate(1, 20));
//    snprintf(str, 40, "XDE %x XHL %x", regL(2), regL(3));
//    slPrint(str, slLocate(1, 21));

//    snprintf(str, 40, "bios vector: %x", loadL(0xFFFE14));
//    slPrint(str, slLocate(5, 18));

//    snprintf(str, 40, "cfb: %x, contr: %x", cfb, ram[0x6F82]);
//    slPrint(str, slLocate(5, 19));
#if 0    
    snprintf(str, 40, "bgcol: %x tile map data:", *(_u16*)(_u8*)(ram + 0x83E0 + ((bgc & 7) << 1)));
    slPrint(str, slLocate(5, 20));
    
    snprintf(str, 40, "%x %x %x %x %x %x", loadW(0x9000), loadW(0x9002), loadW(0x9004), loadW(0x9006), loadW(0x9008), loadW(0x900a));
    slPrint(str, slLocate(1, 21));
    snprintf(str, 40, "%x %x %x %x %x %x", loadW(0x900c), loadW(0x900e), loadW(0x9010), loadW(0x9012), loadW(0x9014), loadW(0x9016));
    slPrint(str, slLocate(1, 22));
    snprintf(str, 40, "%x %x %x %x %x %x", loadW(0x9018), loadW(0x901a), loadW(0x901c), loadW(0x901e), loadW(0x9020), loadW(0x9022));
    slPrint(str, slLocate(1, 23));
    
    snprintf(str, 40, "timer_hint %i scanline %i or %i", timer_hint, scanline, loadB(0x8009));
    slPrint(str, slLocate(1, 24));

    snprintf(str, 40, "frameskip_count %i fsk_key %i", frameskip_count, system_frameskip_key);
    slPrint(str, slLocate(1, 25));
#else    
//    printMemory16(0x9000 + 5*6*2, 0, 20, 6, 10);
#endif
}

/*! Called at the start of the vertical blanking period, this function is
	designed to perform many of the critical hardware interface updates
	Here is a list of recommended actions to take:
	
	- The frame buffer should be copied to the screen.
	- The frame rate should be throttled to 59.95hz
	- The sound chips should be polled for the next chunk of data
	- Input should be polled and the current status written to "ram[0x6F82]" */
	
void system_VBL(void)
{    
    EMST_start(EMST_PAD);
    update_inputs();
    EMST_stop();
    
	// Update Sound
	mute = TRUE;

    EMST_frames(1);
    if(frameskip_count == 0) {
        EMST_start(EMST_VRAM);
        // need frameskip management??
        update_graphics();
    
//    clearText();
//    if((Smpc_Peripheral[0].data & PER_DGT_ST) == 0) 
//        printDebug();
//    else printMemory16(0x9000, 0, 0, 6, 29);
        EMST_display();
        slPrintHex(z80_enable, slLocate(15, 25));
        slPrintHex(sound_enable, slLocate(15, 26));
        slPrintHex(system_frameskip_key, slLocate(15, 27));
        slPrintHex(frame, slLocate(15, 28));

        slSynch();
        EMST_stop();
    }
    frame++;    
}

/*! The system sets this value to the number of the frameskip keyframe */

_u8 system_frameskip_key;

//-----------------------------------------------------------------------------
// Core <--> System-Sound Interface
//-----------------------------------------------------------------------------

	// Speed of DAC playback
#define DAC_FREQUENCY		8000 //hz

BOOL mute;

/*!	Fills the given buffer with sound data */

//void sound_update(_u16* chip_buffer, int length_bytes){}

//void dac_update(_u8* dac_buffer, int length_bytes){}

/*! Initialises the sound chips using the given SampleRate */
	
//void sound_init(int SampleRate){}


		//=========================================

/*! Callback for "sound_init" with the system sound frequency */
	
void system_sound_chipreset(void)
{
}

/*! Clears the sound output. */
	
void system_sound_silence(void)
{
}


/*! Reads a byte from the other system. If no data is available or no
	high-level communications have been established, then return FALSE.
	If buffer is NULL, then no data is read, only status is returned */

BOOL system_comms_read(_u8* buffer) {
    return FALSE;
}

/*! Peeks at any data from the other system. If no data is available or
	no high-level communications have been established, then return FALSE.
	If buffer is NULL, then no data is read, only status is returned */

BOOL system_comms_poll(_u8* buffer) {
    return FALSE;    
}

/*! Writes a byte from the other system. This function should block until
	the data is written. USE RELIABLE COMMS! Data cannot be re-requested. */

void system_comms_write(_u8 data) {}


/*! Reads the "appropriate" (system specific) flash data into the given
	preallocated buffer. The emulation core doesn't care where from. */

BOOL system_io_flash_read(_u8* buffer, _u32 bufferLength) {
    return FALSE;
}


/*! Writes the given flash data into an "appropriate" (system specific)
	place. The emulation core doesn't care where to. */

BOOL system_io_flash_write(_u8* buffer, _u32 bufferLength) {
    return FALSE;
}

/*! Reads from the file specified by 'filename' into the given preallocated
	buffer. This is state data. */

BOOL system_io_state_read(char* filename, _u8* buffer, _u32 bufferLength) {
    return FALSE;
}
	

/*! Writes to the file specified by 'filename' from the given buffer.
	This is state data. */

BOOL system_io_state_write(char* filename, _u8* buffer, _u32 bufferLength) {
    return FALSE;
}


