#include <SGL.H>
#include "neopop.h"
#include "were_here.h"

/*! Get a string that may possibly be translated */
char* system_get_string(STRINGS string_id){}

/*! Used to generate a critical message for the user. After the message
	has been displayed, the function should return. The message is not
	necessarily a fatal error. */
	
//	void __cdecl system_message(char* vaMessage,...);
void system_message(char* vaMessage,...) {}


void update_inputs()
{
   	//Clear the controller status
	unsigned char up, down, left, right, button_a, button_b, option;
	Uint16 data = Smpc_Peripheral[0].data;
	
   	//Clear the controller status
	up = down = left = right = button_a = button_b = option = 0;

        // buttons
    if((data & PER_DGT_TA) == 0)
        button_a = 1; 
    if((data & PER_DGT_TB) == 0)
        button_b = 1; 
    if((data & PER_DGT_TC) == 0)
        option = 1; 
        // D-pad
    if((data & PER_DGT_KL) == 0)
        left = 1; 
    if((data & PER_DGT_KR) == 0)
        right = 1; 
    if((data & PER_DGT_KU) == 0)
        up = 1; 
    if((data & PER_DGT_KD) == 0)
        down = 1; 
	   //Write the controller status to memory
	ram[0x6F82] = up | (down << 1) | (left << 2) | (right << 3) | 
		(button_a << 4) | (button_b << 5) | (option << 6);
}


void update_graphics()
{
    int x, y;
	_u16* row = cfb;
    Uint16 *VRAM = (Uint16 *)VDP2_VRAM_A0;
    
    //16 BIT DISPLAY
    for (y = 0; y < SCREEN_HEIGHT; y++) {
        for (x = 0; x < SCREEN_WIDTH; x++) {
            // B G R sample from SL_DEF.H:
            // CD_Blue		(31 << 10) | ( 0 << 5) | (0)  | RGB_Flag
            VRAM[x] = (((row[x] & 0xF00) << (10 - 8)) | ((row[x] & 0x0F0) << (5 - 4)) | ((row[x] & 0x00F))) | RGB_Flag;
        }
		VRAM += 512;
		row += SCREEN_WIDTH;
	}
}

#include "Z80_interface.h"
#include "TLCS900h_registers.h"
#include "TLCS900h_disassemble.h"
extern unsigned int cycle;

void printDebug()
{
    char str[41], *str2;
    _u32 pc_temp;
    
    // debug info:
    clearText();
    slPrint("debug info:", slLocate(5, 5));
        // cartridge name
    snprintf(str, 40, "running cart: %s", rom.name);
    slPrint(str, slLocate(5, 6));
    snprintf(str, 40, "of size: %i at %x", rom.length, rom.data);
    slPrint(str, slLocate(5, 7));
        // cycle count
    snprintf(str, 40, "start PC %x cycle: %i", rom_header->startPC, cycle);
    slPrint(str, slLocate(5, 8));
        // register status
//    snprintf(str, 40, "PS %x A %x SP %x", regs.PS, regs.A, regs.SP);
//    slPrint(str, slLocate(5, 8));
    snprintf(str, 40, "Z80 PC %x TLCS900 PC %x", Z80_getReg(Z80_REG_PC), pc);
    slPrint(str, slLocate(5, 9));
// TLCS900h_disassemble():
//Disassembles a single instruction from 'pc', 
//pc is incremented to the start of the next instruction.
    pc_temp = pc;
    str2 = TLCS900h_disassemble();
    slPrint(str2, slLocate(2, 10));
    slPrint(str2 + 20, slLocate(2, 11));
    pc = pc_temp;
//    snprintf(str, 40, "boot: %x, button: %x", gCPUBootAddress, newsystem->GetButtonData());
//    slPrint(str, slLocate(5, 11));
    
}

/*! Called at the start of the vertical blanking period, this function is
	designed to perform many of the critical hardware interface updates
	Here is a list of recommended actions to take:
	
	- The frame buffer should be copied to the screen.
	- The frame rate should be throttled to 59.95hz
	- The sound chips should be polled for the next chunk of data
	- Input should be polled and the current status written to "ram[0x6F82]" */
	
void system_VBL(void)
{
    // need frameskip management??
    update_graphics();
    
    printDebug();
    
    update_inputs();
    
	// Update Sound
	mute = TRUE;
	
    // framerate estimate and throttle??
    system_frameskip_key = 0;
    
    slSynch();
}

/*! The system sets this value to the number of the frameskip keyframe */

_u8 system_frameskip_key;

//-----------------------------------------------------------------------------
// Core <--> System-Sound Interface
//-----------------------------------------------------------------------------

	// Speed of DAC playback
#define DAC_FREQUENCY		8000 //hz

BOOL mute;

/*!	Fills the given buffer with sound data */

//void sound_update(_u16* chip_buffer, int length_bytes){}

//void dac_update(_u8* dac_buffer, int length_bytes){}

/*! Initialises the sound chips using the given SampleRate */
	
//void sound_init(int SampleRate){}


		//=========================================

/*! Callback for "sound_init" with the system sound frequency */
	
void system_sound_chipreset(void)
{
}

/*! Clears the sound output. */
	
void system_sound_silence(void)
{
}


/*! Reads a byte from the other system. If no data is available or no
	high-level communications have been established, then return FALSE.
	If buffer is NULL, then no data is read, only status is returned */

BOOL system_comms_read(_u8* buffer){}

/*! Peeks at any data from the other system. If no data is available or
	no high-level communications have been established, then return FALSE.
	If buffer is NULL, then no data is read, only status is returned */

BOOL system_comms_poll(_u8* buffer){}

/*! Writes a byte from the other system. This function should block until
	the data is written. USE RELIABLE COMMS! Data cannot be re-requested. */

void system_comms_write(_u8 data){}


/*! Reads the "appropriate" (system specific) flash data into the given
	preallocated buffer. The emulation core doesn't care where from. */

BOOL system_io_flash_read(_u8* buffer, _u32 bufferLength){}


/*! Writes the given flash data into an "appropriate" (system specific)
	place. The emulation core doesn't care where to. */

BOOL system_io_flash_write(_u8* buffer, _u32 bufferLength){}

/*! Reads from the file specified by 'filename' into the given preallocated
	buffer. This is state data. */

BOOL system_io_state_read(char* filename, _u8* buffer, _u32 bufferLength){}
	

/*! Writes to the file specified by 'filename' from the given buffer.
	This is state data. */

BOOL system_io_state_write(char* filename, _u8* buffer, _u32 bufferLength){}


